@@ -19,8 +19,6 @@ single line if there were no line length restrictions. It's then used by the
 parser to perform the wrapping required to comply with the style guide.
 """
 
-from lib2to3 import pytree
-
 from yapf.yapflib import format_token
 from yapf.yapflib import py3compat
 from yapf.yapflib import pytree_utils
@@ -233,7 +231,7 @@ def _SpaceRequiredBetween(left, right):
     # A string followed by something other than a subscript, closing bracket,
     # or dot should have a space after it.
     return True
-  if left.is_binary_op and _IsUnaryOperator(right):
+  if left.is_binary_op and lval != '**' and _IsUnaryOperator(right):
     # Space between the binary opertor and the unary operator.
     return True
   if _IsUnaryOperator(left) and _IsUnaryOperator(right):
@@ -257,7 +255,8 @@ def _SpaceRequiredBetween(left, right):
   if (format_token.Subtype.DEFAULT_OR_NAMED_ASSIGN in left.subtypes or
       format_token.Subtype.DEFAULT_OR_NAMED_ASSIGN in right.subtypes):
     # A named argument or default parameter shouldn't have spaces around it.
-    return False
+    # However, a typed argument should have a space after the colon.
+    return lval == ':' or style.Get('SPACES_AROUND_DEFAULT_OR_NAMED_ASSIGN')
   if (format_token.Subtype.VARARGS_STAR in left.subtypes or
       format_token.Subtype.KWARGS_STAR_STAR in left.subtypes):
     # Don't add a space after a vararg's star or a keyword's star-star.
@@ -326,9 +325,8 @@ def _MustBreakBefore(prev_token, cur_token):
     # reasonable assumption, because otherwise they should have written them
     # all on the same line, or with a '+'.
     return True
-  return pytree_utils.GetNodeAnnotation(cur_token.node,
-                                        pytree_utils.Annotation.MUST_SPLIT,
-                                        default=False)
+  return pytree_utils.GetNodeAnnotation(
+      cur_token.node, pytree_utils.Annotation.MUST_SPLIT, default=False)
 
 
 def _CanBreakBefore(prev_token, cur_token):
