@@ -26,6 +26,7 @@ import unittest
 
 from yapf.yapflib import py3compat
 from yapf.yapflib import yapf_api
+from yapf.yapflib import style
 
 ROOT_DIR = os.path.dirname(os.path.abspath(os.path.dirname(__file__)))
 
@@ -36,8 +37,8 @@ YAPF_BINARY = [sys.executable, '-m', 'yapf', '--verify', '--no-local-style']
 class FormatCodeTest(unittest.TestCase):
 
   def _Check(self, unformatted_code, expected_formatted_code):
-    formatted_code, _ = yapf_api.FormatCode(unformatted_code,
-                                            style_config='chromium')
+    formatted_code, _ = yapf_api.FormatCode(
+        unformatted_code, style_config='chromium')
     self.assertEqual(expected_formatted_code, formatted_code)
 
   def testSimple(self):
@@ -210,9 +211,8 @@ class FormatFileTest(unittest.TestCase):
 
         if h:    i
         """)
-    formatted_code = yapf_api.FormatFile(file1,
-                                         style_config='pep8',
-                                         lines=[(1, 2)])[0]
+    formatted_code = yapf_api.FormatFile(
+        file1, style_config='pep8', lines=[(1, 2)])[0]
     self.assertCodeEqual(expected_formatted_code_lines1and2, formatted_code)
 
     expected_formatted_code_lines3 = textwrap.dedent(u"""\
@@ -222,9 +222,8 @@ class FormatFileTest(unittest.TestCase):
 
         if h:    i
         """)
-    formatted_code = yapf_api.FormatFile(file1,
-                                         style_config='pep8',
-                                         lines=[(3, 3)])[0]
+    formatted_code = yapf_api.FormatFile(
+        file1, style_config='pep8', lines=[(3, 3)])[0]
     self.assertCodeEqual(expected_formatted_code_lines3, formatted_code)
 
   def testFormatFileDiff(self):
@@ -245,21 +244,16 @@ class FormatFileTest(unittest.TestCase):
     with open(file1) as f:
       self.assertCodeEqual(formatted_code, f.read())
 
-    self.assertRaises(ValueError,
-                      yapf_api.FormatFile,
-                      file1,
-                      in_place=True,
-                      print_diff=True)
+    self.assertRaises(
+        ValueError, yapf_api.FormatFile, file1, in_place=True, print_diff=True)
 
   def testNoFile(self):
     stream = py3compat.StringIO()
     handler = logging.StreamHandler(stream)
     logger = logging.getLogger('mylogger')
     logger.addHandler(handler)
-    self.assertRaises(IOError,
-                      yapf_api.FormatFile,
-                      'not_a_file.py',
-                      logger=logger.error)
+    self.assertRaises(
+        IOError, yapf_api.FormatFile, 'not_a_file.py', logger=logger.error)
     self.assertEqual(stream.getvalue(),
                      "[Errno 2] No such file or directory: 'not_a_file.py'\n")
 
@@ -295,6 +289,25 @@ class FormatFileTest(unittest.TestCase):
     formatted_code = yapf_api.FormatFile(file1, style_config='pep8')[0]
     self.assertCodeEqual(code, formatted_code)
 
+  def testDisabledMultilineStringInDictionary(self):
+    code = textwrap.dedent("""\
+        # yapf: disable
+
+        A = [
+            {
+                "aaaaaaaaaaaaaaaaaaa": '''
+        bbbbbbbbbbb: "ccccccccccc"
+        dddddddddddddd: 1
+        eeeeeeee: 0
+        ffffffffff: "ggggggg"
+        ''',
+            },
+        ]
+        """)
+    file1 = self._MakeTempFileWithContents('testfile1.py', code)
+    formatted_code = yapf_api.FormatFile(file1, style_config='chromium')[0]
+    self.assertCodeEqual(code, formatted_code)
+
 
 class CommandLineTest(unittest.TestCase):
   """Test how calling yapf from the command line acts."""
@@ -319,10 +332,11 @@ class CommandLineTest(unittest.TestCase):
       extra_options: iterable of extra command-line options to pass to yapf
     """
     cmdline = YAPF_BINARY + (extra_options or [])
-    p = subprocess.Popen(cmdline,
-                         stdout=subprocess.PIPE,
-                         stdin=subprocess.PIPE,
-                         stderr=subprocess.PIPE)
+    p = subprocess.Popen(
+        cmdline,
+        stdout=subprocess.PIPE,
+        stdin=subprocess.PIPE,
+        stderr=subprocess.PIPE)
     reformatted_code, stderrdata = p.communicate(unformatted.encode('utf-8'))
     self.assertEqual(stderrdata, b'')
     self.assertEqual(reformatted_code.decode('utf-8'), expected)
@@ -333,13 +347,13 @@ class CommandLineTest(unittest.TestCase):
             print('⇒')
         """)
 
-    with tempfile.NamedTemporaryFile(suffix='.py',
-                                     dir=self.test_tmpdir) as outfile:
-      with tempfile.NamedTemporaryFile(suffix='.py',
-                                       dir=self.test_tmpdir) as testfile:
+    with tempfile.NamedTemporaryFile(
+        suffix='.py', dir=self.test_tmpdir) as outfile:
+      with tempfile.NamedTemporaryFile(
+          suffix='.py', dir=self.test_tmpdir) as testfile:
         testfile.write(unformatted_code.encode('UTF-8'))
-        subprocess.check_call(YAPF_BINARY + ['--diff', testfile.name],
-                              stdout=outfile)
+        subprocess.check_call(
+            YAPF_BINARY + ['--diff', testfile.name], stdout=outfile)
 
   def testInPlaceReformatting(self):
     unformatted_code = textwrap.dedent(u"""\
@@ -351,8 +365,8 @@ class CommandLineTest(unittest.TestCase):
             x = 37
         """)
 
-    with tempfile.NamedTemporaryFile(suffix='.py',
-                                     dir=self.test_tmpdir) as testfile:
+    with tempfile.NamedTemporaryFile(
+        suffix='.py', dir=self.test_tmpdir) as testfile:
       testfile.write(unformatted_code.encode('UTF-8'))
       testfile.seek(0)
 
@@ -368,8 +382,8 @@ class CommandLineTest(unittest.TestCase):
     unformatted_code = u'\n\n'
     expected_formatted_code = u'\n'
 
-    with tempfile.NamedTemporaryFile(suffix='.py',
-                                     dir=self.test_tmpdir) as testfile:
+    with tempfile.NamedTemporaryFile(
+        suffix='.py', dir=self.test_tmpdir) as testfile:
       testfile.write(unformatted_code.encode('UTF-8'))
       testfile.seek(0)
 
@@ -385,8 +399,8 @@ class CommandLineTest(unittest.TestCase):
     unformatted_code = u''
     expected_formatted_code = u''
 
-    with tempfile.NamedTemporaryFile(suffix='.py',
-                                     dir=self.test_tmpdir) as testfile:
+    with tempfile.NamedTemporaryFile(
+        suffix='.py', dir=self.test_tmpdir) as testfile:
       testfile.write(unformatted_code.encode('UTF-8'))
       testfile.seek(0)
 
@@ -427,9 +441,10 @@ class CommandLineTest(unittest.TestCase):
         def foo():  # trail
           x = 37
         """)
-    self.assertYapfReformats(unformatted_code,
-                             expected_formatted_code,
-                             extra_options=['--style=chromium'])
+    self.assertYapfReformats(
+        unformatted_code,
+        expected_formatted_code,
+        extra_options=['--style=chromium'])
 
   def testSetCustomStyleBasedOnChromium(self):
     unformatted_code = textwrap.dedent(u"""\
@@ -442,15 +457,17 @@ class CommandLineTest(unittest.TestCase):
         """)
 
     with tempfile.NamedTemporaryFile(dir=self.test_tmpdir, mode='w') as f:
-      f.write(textwrap.dedent('''\
+      f.write(
+          textwrap.dedent('''\
           [style]
           based_on_style = chromium
           SPACES_BEFORE_COMMENT = 4
           '''))
       f.flush()
-      self.assertYapfReformats(unformatted_code,
-                               expected_formatted_code,
-                               extra_options=['--style={0}'.format(f.name)])
+      self.assertYapfReformats(
+          unformatted_code,
+          expected_formatted_code,
+          extra_options=['--style={0}'.format(f.name)])
 
   def testReadSingleLineCodeFromStdin(self):
     unformatted_code = textwrap.dedent(u"""\
@@ -469,13 +486,13 @@ class CommandLineTest(unittest.TestCase):
             x = 37
         """)
 
-    with tempfile.NamedTemporaryFile(suffix='.py',
-                                     dir=self.test_tmpdir) as outfile:
-      with tempfile.NamedTemporaryFile(suffix='.py',
-                                       dir=self.test_tmpdir) as testfile:
+    with tempfile.NamedTemporaryFile(
+        suffix='.py', dir=self.test_tmpdir) as outfile:
+      with tempfile.NamedTemporaryFile(
+          suffix='.py', dir=self.test_tmpdir) as testfile:
         testfile.write(unformatted_code.encode('utf-8'))
-        subprocess.check_call(YAPF_BINARY + ['--diff', testfile.name],
-                              stdout=outfile)
+        subprocess.check_call(
+            YAPF_BINARY + ['--diff', testfile.name], stdout=outfile)
 
   def testReformattingSpecificLines(self):
     unformatted_code = textwrap.dedent(u"""\
@@ -502,9 +519,10 @@ class CommandLineTest(unittest.TestCase):
     # TODO(ambv): the `expected_formatted_code` here is not PEP8 compliant,
     # raising "E129 visually indented line with same indent as next logical
     # line" with flake8.
-    self.assertYapfReformats(unformatted_code,
-                             expected_formatted_code,
-                             extra_options=['--lines', '1-2'])
+    self.assertYapfReformats(
+        unformatted_code,
+        expected_formatted_code,
+        extra_options=['--lines', '1-2'])
 
   def testReformattingSkippingLines(self):
     unformatted_code = textwrap.dedent(u"""\
@@ -674,9 +692,10 @@ class CommandLineTest(unittest.TestCase):
 
                 pass
         """)
-    self.assertYapfReformats(unformatted_code,
-                             expected_formatted_code,
-                             extra_options=['--lines', '1-2'])
+    self.assertYapfReformats(
+        unformatted_code,
+        expected_formatted_code,
+        extra_options=['--lines', '1-2'])
 
     unformatted_code = textwrap.dedent(u"""\
 
@@ -724,9 +743,8 @@ class CommandLineTest(unittest.TestCase):
         import blah
         """)
 
-    self.assertYapfReformats(unformatted_code,
-                             unformatted_code,
-                             extra_options=['--lines', '2-2'])
+    self.assertYapfReformats(
+        unformatted_code, unformatted_code, extra_options=['--lines', '2-2'])
 
   def testRetainingSemicolonsWhenSpecifyingLines(self):
     unformatted_code = textwrap.dedent("""\
@@ -741,9 +759,10 @@ class CommandLineTest(unittest.TestCase):
             x = y + 42; z = n * 42
             if True: a += 1 ; b += 1 ; c += 1
         """)
-    self.assertYapfReformats(unformatted_code,
-                             expected_formatted_code,
-                             extra_options=['--lines', '1-1'])
+    self.assertYapfReformats(
+        unformatted_code,
+        expected_formatted_code,
+        extra_options=['--lines', '1-1'])
 
   def testDisabledMultilineStrings(self):
     unformatted_code = textwrap.dedent('''\
@@ -766,9 +785,10 @@ class CommandLineTest(unittest.TestCase):
         </body>
         </html>"""
         ''')
-    self.assertYapfReformats(unformatted_code,
-                             expected_formatted_code,
-                             extra_options=['--lines', '1-1'])
+    self.assertYapfReformats(
+        unformatted_code,
+        expected_formatted_code,
+        extra_options=['--lines', '1-1'])
 
   def testDisableWhenSpecifyingLines(self):
     unformatted_code = textwrap.dedent(u"""\
@@ -795,9 +815,10 @@ class CommandLineTest(unittest.TestCase):
             'world',
         ])  # yapf: disable
         """)
-    self.assertYapfReformats(unformatted_code,
-                             expected_formatted_code,
-                             extra_options=['--lines', '1-10'])
+    self.assertYapfReformats(
+        unformatted_code,
+        expected_formatted_code,
+        extra_options=['--lines', '1-10'])
 
   def testDisableFormattingInDataLiteral(self):
     unformatted_code = textwrap.dedent(u"""\
@@ -834,9 +855,10 @@ class CommandLineTest(unittest.TestCase):
             why_would_you()
             ['do', 'that']
         """)
-    self.assertYapfReformats(unformatted_code,
-                             expected_formatted_code,
-                             extra_options=['--lines', '14-15'])
+    self.assertYapfReformats(
+        unformatted_code,
+        expected_formatted_code,
+        extra_options=['--lines', '14-15'])
 
   def testRetainVerticalFormattingBetweenDisabledAndEnabledLines(self):
     unformatted_code = textwrap.dedent(u"""\
@@ -859,9 +881,10 @@ class CommandLineTest(unittest.TestCase):
                                  gggggggggggg.hhhhhhhhh(c, c.ffffffffffff))
                 iiiii = jjjjjjjjjjjjjj.iiiii
         """)
-    self.assertYapfReformats(unformatted_code,
-                             expected_formatted_code,
-                             extra_options=['--lines', '4-7'])
+    self.assertYapfReformats(
+        unformatted_code,
+        expected_formatted_code,
+        extra_options=['--lines', '4-7'])
 
   def testFormatLinesSpecifiedInMiddleOfExpression(self):
     unformatted_code = textwrap.dedent(u"""\
@@ -884,9 +907,10 @@ class CommandLineTest(unittest.TestCase):
                                  gggggggggggg.hhhhhhhhh(c, c.ffffffffffff))
                 iiiii = jjjjjjjjjjjjjj.iiiii
         """)
-    self.assertYapfReformats(unformatted_code,
-                             expected_formatted_code,
-                             extra_options=['--lines', '5-6'])
+    self.assertYapfReformats(
+        unformatted_code,
+        expected_formatted_code,
+        extra_options=['--lines', '5-6'])
 
   def testCommentFollowingMultilineString(self):
     unformatted_code = textwrap.dedent(u"""\
@@ -905,9 +929,10 @@ class CommandLineTest(unittest.TestCase):
             x = '''hello world'''  # second comment
             return 42  # another comment
         """)
-    self.assertYapfReformats(unformatted_code,
-                             expected_formatted_code,
-                             extra_options=['--lines', '1-1'])
+    self.assertYapfReformats(
+        unformatted_code,
+        expected_formatted_code,
+        extra_options=['--lines', '1-1'])
 
   def testDedentClosingBracket(self):
     # no line-break on the first argument, not dedenting closing brackets
@@ -921,9 +946,10 @@ class CommandLineTest(unittest.TestCase):
                                     second_argument_makes_the_line_too_long):
           pass
     """)
-    self.assertYapfReformats(unformatted_code,
-                             expected_formatted_code,
-                             extra_options=['--style=pep8'])
+    self.assertYapfReformats(
+        unformatted_code,
+        expected_formatted_code,
+        extra_options=['--style=pep8'])
 
     # TODO(ambv): currently the following produces the closing bracket on a new
     # line but indented to the opening bracket which is the worst of both
@@ -951,9 +977,10 @@ second_argument_makes_the_line_too_long):
       ):
           pass
     """)
-    self.assertYapfReformats(unformatted_code,
-                             expected_formatted_fb_code,
-                             extra_options=['--style=facebook'])
+    self.assertYapfReformats(
+        unformatted_code,
+        expected_formatted_fb_code,
+        extra_options=['--style=facebook'])
     # TODO(ambv): currently the following produces code that is not PEP8
     # compliant, raising "E125 continuation line with same indent as next
     # logical line" with flake8. Expected behaviour for PEP8 would be to use
@@ -961,6 +988,32 @@ second_argument_makes_the_line_too_long):
     # self.assertYapfReformats(unformatted_code, expected_formatted_pep8_code,
     #                          extra_options=['--style=pep8'])
 
+  def testCoalesceBrackets(self):
+    unformatted_code = textwrap.dedent(u"""\
+       some_long_function_name_foo({
+           'first_argument_of_the_thing': id,
+           'second_argument_of_the_thing': "some thing"}
+           )""")
+    expected_formatted_code = textwrap.dedent(u"""\
+       some_long_function_name_foo({
+           'first_argument_of_the_thing': id,
+           'second_argument_of_the_thing': "some thing"
+       })
+       """)
+    with tempfile.NamedTemporaryFile(dir=self.test_tmpdir, mode='w') as f:
+      f.write(
+          textwrap.dedent('''\
+          [style]
+          based_on_style = facebook
+          column_limit=82
+          coalesce_brackets = True
+          '''))
+      f.flush()
+      self.assertYapfReformats(
+          unformatted_code,
+          expected_formatted_code,
+          extra_options=['--style={0}'.format(f.name)])
+
   def testPseudoParenSpaces(self):
     unformatted_code = textwrap.dedent(u"""\
         def foo():
@@ -972,10 +1025,10 @@ second_argument_makes_the_line_too_long):
           def bar():
             return {msg_id: author for author, msg_id in reader}
         """)
-    self.assertYapfReformats(unformatted_code,
-                             expected_formatted_code,
-                             extra_options=['--lines', '1-1', '--style',
-                                            'chromium'])
+    self.assertYapfReformats(
+        unformatted_code,
+        expected_formatted_code,
+        extra_options=['--lines', '1-1', '--style', 'chromium'])
 
   def testMultilineCommentFormattingDisabled(self):
     unformatted_code = textwrap.dedent(u"""\
@@ -1006,10 +1059,10 @@ second_argument_makes_the_line_too_long):
             '#': lambda x: x  # do nothing
         }
         """)
-    self.assertYapfReformats(unformatted_code,
-                             expected_formatted_code,
-                             extra_options=['--lines', '1-1', '--style',
-                                            'chromium'])
+    self.assertYapfReformats(
+        unformatted_code,
+        expected_formatted_code,
+        extra_options=['--lines', '1-1', '--style', 'chromium'])
 
   def testTrailingCommentsWithDisabledFormatting(self):
     unformatted_code = textwrap.dedent(u"""\
@@ -1026,10 +1079,35 @@ second_argument_makes_the_line_too_long):
             'hello world'  # This is a comment.
         ]
         """)
-    self.assertYapfReformats(unformatted_code,
-                             expected_formatted_code,
-                             extra_options=['--lines', '1-1', '--style',
-                                            'chromium'])
+    self.assertYapfReformats(
+        unformatted_code,
+        expected_formatted_code,
+        extra_options=['--lines', '1-1', '--style', 'chromium'])
+
+  def testUseTabs(self):
+    unformatted_code = textwrap.dedent(u"""\
+        def foo_function():
+         if True:
+          pass
+        """)
+    expected_formatted_code = textwrap.dedent(u"""\
+        def foo_function():
+        	if True:
+        		pass
+        """)
+    with tempfile.NamedTemporaryFile(dir=self.test_tmpdir, mode='w') as f:
+      f.write(
+          textwrap.dedent('''\
+          [style]
+          based_on_style = chromium
+          USE_TABS = true
+          INDENT_WIDTH=1
+          '''))
+      f.flush()
+      self.assertYapfReformats(
+          unformatted_code,
+          expected_formatted_code,
+          extra_options=['--style={0}'.format(f.name)])
 
 
 class BadInputTest(unittest.TestCase):
@@ -1040,5 +1118,31 @@ class BadInputTest(unittest.TestCase):
     self.assertRaises(SyntaxError, yapf_api.FormatCode, code)
 
 
+class DiffIndentTest(unittest.TestCase):
+
+  @staticmethod
+  def own_style():
+    my_style = style.CreatePEP8Style()
+    my_style['INDENT_WIDTH'] = 3
+    my_style['CONTINUATION_INDENT_WIDTH'] = 3
+    return my_style
+
+  def _Check(self, unformatted_code, expected_formatted_code):
+    formatted_code, _ = yapf_api.FormatCode(
+        unformatted_code, style_config=style.SetGlobalStyle(self.own_style()))
+    self.assertEqual(expected_formatted_code, formatted_code)
+
+  def testSimple(self):
+    unformatted_code = textwrap.dedent(u"""\
+        for i in range(5):
+         print('bar')
+         """)
+    expected_formatted_code = textwrap.dedent(u"""\
+        for i in range(5):
+           print('bar')
+           """)
+    self._Check(unformatted_code, expected_formatted_code)
+
+
 if __name__ == '__main__':
   unittest.main()
